program Amp_check_M
  use sep

! Seth Haines, Jan 2003 (Dipole), altered to make Amp_check Sept 2003
!              oct 03  altered again so I can input the dipole radius instead of using spreading
!                       for doing source dipole modeling
!              April 04- altered to make Amp_check_M for the metal plate effect- a dipole inline with 
!                          electrode array

! inputs: H file w/ proper dimensions for output, dominant frequency (f0), source-interface distance (xs), 
!          rec-interface distance (xr),
!
! output: a 1-D vector (in an H file) of the amplitudes for the given geometry
!


implicit none
  integer                          :: n1, n2, i1, i2, j
  real                             :: d1, d2, pi, eps, const, L, R, add, elec_width, elec_rad, q, s,\
                                      y, x, o2, dip_rad, lamda, yr, xr, y1, y2
  real, dimension (:), allocatable ::  amps

call sep_init()
  call from_history("n1",n1)
  call from_history("n2",n2)
  call from_history("d1",d1)
  call from_history("d2",d2)
  call from_history("o2",o2)
  call from_param("xr",xr,0.)
  call from_param("yr",yr,0.)
  call from_param("q",q)

  call to_history("n1",n2)
  call to_history("n2",1)
  call to_history("n3",1)
  call to_history("d1",d2)
  call to_history("o1",o2)
  call to_history ("label1","offset (m)")
  call to_history ("label2","amplitude")

  write(0,*) 'dipole offset from zero by distance ',yr
  write(0,*) 'Receiver-dipole offset is ', xr


  pi=3.141592654
  eps=8.85E-12   !elec perm
!  q=.00000007            !charge - now it's input from param
  s=0.2         !spacing between poles of dipole
  const=(q*s)/(4*pi*eps)
  write(0,*) 'constant is', const
  elec_width=1.0         !width of electrode dipole
  elec_rad=elec_width/2    !"radius" of electrode dipole

allocate (amps (n2))
amps=0

 offset: do i2 = 1, n2
          y1 = (i2-1)*d2 + o2 - elec_rad + yr
          y2 = (i2-1)*d2 + o2 + elec_rad + yr
!write(0,*) 'x is ', x
       L=(const*y1)/((xr*xr + y1*y1)**1.5)
       R=(const*y2)/((xr*xr + y2*y2)**1.5)

!       L=(const*xr)/(((((x-dip_rad)+(j*dip_space))-elec_rad)*(((x-dip_rad)+(j*dip_space))-elec_rad))+xr*xr)**1.5  !old
!       R=(const*xr)/(((((x-dip_rad)+(j*dip_space))+elec_rad)*(((x-dip_rad)+(j*dip_space))+elec_rad))+xr*xr)**1.5  !old

!       write(0,*) 'L is ', L
!       write(0,*) 'R is ', R
       add=(L-R)   !difference between potentials (signal measured by seismograph)
!write(0,*) 'here'
!write(0,*) 'add is ', add
       amps(i2)=amps(i2)+add     !sum up all the little electrodes
   end do offset


call sep_write(amps)
call sep_close ()

deallocate (amps)
call exit (0)
end program Amp_check_M
