program Amp_check
  use sep

! Seth Haines, Jan 2003 (Dipole), altered to make Amp_check Sept 2003
!              oct 03  altered again so I can input the dipole radius instead of using spreading
!                       for doing source dipole modeling
! inputs: velocity model, dominant frequency (f0), source-interface distance (xs), 
!          rec-interface distance (xr), number of dipoles (no_dips) (gotta be at least 3 or something),
!
! output: a 1-D vector (in an H file) of the amplitudes for the given geometry
!


implicit none
  integer                          :: n1, n2, i1, i2, j, no_dips
  real                             :: d1, d2, pi, f0, eps, const, L, R, add, dip_space, elec_width, elec_rad, q, s,\
                                      y, x, o2, dip_rad, lamda, xs, xr, r_dip
  real, dimension (:, :), allocatable :: vels
  real, dimension (:), allocatable ::  amps

call sep_init()
  call from_history("n1",n1)
  call from_history("n2",n2)
  call from_history("d1",d1)
  call from_history("d2",d2)
  call from_history("o2",o2)
  call from_param("f0",f0)
  call from_param("no_dips",no_dips)
  call from_param("xs",xs)
  call from_param("xr",xr)
  call from_param("r_dip",r_dip)
  call from_param("q",q)

  call to_history("n1",n2)
  call to_history("n2",1)
  call to_history("n3",1)
  call to_history("d1",d2)
  call to_history("o1",o2)
  call to_history ("label1","offset (m)")
  call to_history ("label2","amplitude")

  write(0,*) 'Source-interface distance is ', xs
  write(0,*) 'Receiver-interface distance is ', xr
  write(0,*) 'Dipole radius is ', r_dip



  pi=3.141592654
  eps=8.85E-12   !elec perm
!  q=.00000007            !charge - now it's input from param
  s=0.0001         !spacing between poles of dipole
  const=(q*s)/(4*pi*eps)
  write(0,*) 'constant is', const
  elec_width=1.0         !width of electrode dipole
  elec_rad=elec_width/2    !"radius" of electrode dipole

allocate (vels (n1,n2))
allocate (amps (n2))
call sep_read (vels)
amps=0

!y=(vels(i1,(n2/2))*i1*d1)   !remnant from Dipole.f90- calculates depth to interface (velocity*time)

i1=1
findit: do while ((vels(i1,(n2/2))*i1*d1) < xs)
i1=i1+1
end do findit

lamda=(vels(i1,(n2/2))/f0)       !wavelength (at interface) of dominant frequency of seismic
!dip_rad=sqrt((lamda*xs)/2)       !OLD:  radius of first Fresnel zone, using Sherrif and Geldart's eqn
dip_rad=r_dip                    !New:  entered by hand

dip_space=((2*dip_rad)/(no_dips-1))  !spacing between each little dipole
 offset: do i2 = 1, n2
    dips: do j = 0, (no_dips-1)
       x=(i2-1)*d2+o2
!write(0,*) 'x is ', x
       L=(const*xr)/(((((x-dip_rad)+(j*dip_space))-elec_rad)*(((x-dip_rad)+(j*dip_space))-elec_rad))+xr*xr)**1.5  !potential at one electrode
       R=(const*xr)/(((((x-dip_rad)+(j*dip_space))+elec_rad)*(((x-dip_rad)+(j*dip_space))+elec_rad))+xr*xr)**1.5  !potential at the other electrode
!       write(0,*) 'L is ', L
!       write(0,*) 'R is ', R
       add=(L-R)/no_dips   !difference between potentials (signal measured by seismograph), normalized
!write(0,*) 'here'
!write(0,*) 'add is ', add
       amps(i2)=amps(i2)+add     !sum up all the little electrodes
    end do dips
 end do offset


call sep_write(amps)
call sep_close ()

deallocate (vels)
deallocate (amps)
call exit (0)
end program Amp_check
