program Circle_amps

!April 2004  Seth Haines
!calculate amplitudes to be observed by the circle electrode deployment
!due to a horizontal electric dipole (or equivalent)
!assumes 12 radial channels and 12 transverse

!all angles are in degrees!!

!inputs: r_circ: circle radius, beta: angle (degrees) between dipole and ch 1
!        r_elec: half-width of electrode pairs, q: elec charge of poles of dipole
!output: H file w/ 12 radial values and then 12 transverse

use sep

implicit none
  integer                     :: n1,i1,nchan
  real                        :: eps, pi, const, r_circ, beta, theta, r_elec, r1, r2, t1, t2, q, s, d1
  real                        :: rpot1,rpot2,tpot1,tpot2
  real,allocatable,dimension (:)   :: rad,trans


call sep_init()
  call from_param("r_circ",r_circ)
  call from_param("beta",beta)
  call from_param("r_elec",r_elec)
  call from_param("q",q)

n1=24     !better be even
nchan=n1/2
d1=360/nchan
  call to_history("n1",n1)
  call to_history("d1",d1)
  call to_history("o1",0)
  call to_history("label1","position (deg)")

pi=3.141592654
eps=8.85E-12      ! elec perm
!q=0.000002         ! charge, now it's input from param, though
s=0.01            ! spacing between poles of dipole
const = (q*s)/(4*pi*eps)
allocate(rad(nchan), trans(nchan))

angles: do i1 = 1,nchan
   theta = (i1-1)*d1 + beta
   r1 = r_circ - r_elec
   r2 = r_circ + r_elec
  !radial potentials
   rpot1 = const*r1*cosd(theta)/(r1*r1*r1)
   rpot2 = const*r2*cosd(theta)/(r2*r2*r2)
   rad(i1) = rpot2 - rpot1
  !transverse potentials
   t1 = theta - asind(r_elec/r_circ)
   t2 = theta + asind(r_elec/r_circ)
   tpot1 = const*r_circ*cosd(t1)/(r_circ*r_circ*r_circ)
   tpot2 = const*r_circ*cosd(t2)/(r_circ*r_circ*r_circ)
   trans(i1) = tpot2 - tpot1
end do angles


call sep_write(rad)
call sep_write(trans)

call sep_close()
deallocate(rad, trans)
end program Circle_amps
