! ---------------------------------------------------------------------------
!  Harm_sub.f90 - Created by Seth Haines w/ Morgan Brown
!           Fri Jun 13 09:40:30 2003.
! ---------------------------------------------------------------------------
!       Usage: Harm_sub.x 
!  Parameters: 
! Description: based on the 2003 Butler and Russel geophysics paper
! ---------------------------------------------------------------------------

!  NEED TO MAKE IT WORK FOR 3-D!!!!!!!!!!!!!!!!!!

program Harm_sub

use sep
use harm_module
use solver_smp_mod
use cgstep_mod
use dottest
!use scaled_identity

implicit none

integer                     :: n1, n2,n3, niter, dim, bigN, bigM, i2, i3, t0s, tfs, iter, ii, alph_it
real                        :: d1, d2, o1, o2, t0, tf, f0, w, w0, neww, pi, phi, phi_der, phi_2der, err, der, newphi_der
real                        :: der_eps, eps, bigT, alpha, newphi, alpha_eps, alpha_u, alpha_l
real, dimension(:), pointer :: model, d_est, xyz, new_trace, harm, ab, dot1, dot2, newxyz, newmodel
real, dimension(:,:,:), pointer :: ab_mat, data
real, dimension(:,:,:), pointer :: f0_vec
logical                       :: verb

call sep_init(SOURCE)

call from_history('n1',n1)
call from_history('o1',o1)
call from_history('d1',d1)
call from_history('n2',n2)
call from_history('n3',n3)

call from_param('t0',t0)          !start of estimation window, in seconds
call from_param('tf',tf)          !end of estimation window, in seconds
call from_param('niter',niter,10)
call from_param('f0',f0, 60.)         !f0 is the starting guess for fundamental freq. in Hz
call from_param('verb',verb,.true.)

bigT=1/(f0*d1)                 ! dominant period in samples (as per the paper)
pi=3.141592654
w0=(2*pi)/bigT                  ! dominant freq in radians
bigM=(1/(2*d1))/f0           ! number of harmonics. find all harmonics up to Nyquist
!t0s=(t0 - o1)/d1               ! starting sample number for estimation window
!tfs=(tf - o1)/d1 
!write(0,*) 'd1 is ', d1
!write(0,*) 't0s should be ', (t0-o1)/d1
!write(0,*) 'tfs should be ', (tf-o1)/d1
t0s=nint(1+(t0-o1)/d1)               ! starting sample number for estimation window
tfs=nint((tf-o1)/d1)               ! ending sample number for estimation window
!write(0,*) 't0s is ', t0s
!write(0,*) 'tfs is ', tfs
bigN=(tfs-t0s+1)               ! number of samples in estimation window
dim=(2*bigM)+1                 ! dimension of the various matrices and shit

allocate( model(dim), d_est(bigN), xyz(dim), ab_mat(dim,n2,n3), f0_vec(2,n2,n3), data(n1,n2,n3) )
allocate( new_trace(n1), harm(n1), ab(dim), newxyz(dim), newmodel(dim) )

call sep_read(data)
call harm_module_init( bigM, bigN, w0 )

!write(0,*) 'o1 is ', o1
write(0,*) 'finding ',bigM,' harmonics of the fundamental'
write(0,*) 'starting fundamental is ', f0, ' Hz'!, or ',w0,' radians'
write(0,*) 'starting estimation at ', t0, ' seconds, sample ',t0s
write(0,*) 'ending estimation at ', tf, ' seconds, sample ',tfs
!write(0,*) 'bigN is ', bigN

!dot test
!allocate(dot1(2), dot2(2))
!call dot_test(harm_module_exec,dim,dim,dot1,dot2)

eps=0.00000001    !for real
!eps=1.         !for no iterations
der_eps=99999.
panels: do i3=1,n3
traces: do i2=1,n2
!   write(0,*) ' '
   write(0,*) ' '
   write(0,*) 'working on trace     ',i2,' of panel ', i3
!   write(0,*) ' '
   d_est=data(t0s:tfs,i2,i3)
   neww=w0
   call reset_L()
   model=0.0          !need a starting guess for the an/bn model
   phi_der=999999.0
!   call find_phi_der(phi_der, d_est, model, w)
   err = 999999.9
   iter=0
   newton: do while(err>eps .and. abs(phi_der)>der_eps)
!   newton: do ii=1,1 
      alpha=5.
      w=neww
      iter = iter + 1
!      write(0,*) ' '
      write(0,*) 'trace     ',i2,' ',i3,' freq ',w/(2*pi*d1),' Hz'
      write(0,*) 'iteration ',iter
      write(0,*) ' '
      xyz=0.0
      call data_init(d_est, xyz, w)
      call solver_smp( model, xyz, harm_module_exec, cgstep, niter, verb=verb )
!      call solver_smp( model, xyz, identity_exec, cgstep, niter, verb=.true. )
      call find_phi(phi, d_est, model, w)
      call find_phi_der(phi_der, d_est, model, w)
      call find_phi_2der(phi_2der, d_est, model, w)
!      neww = w  - (phi/phi_der)
!      neww = w  - (phi_der/phi_2der)  ! newton's method with alpha assumed to be one
      neww = w  - alpha * (phi_der/phi_2der)  ! newton with step size
!      neww = w  - alpha * phi_der  !steepest decent

! begin line search crap, for alpha, modified armijo rule
      call data_init(d_est, newxyz, neww)
      call solver_smp( newmodel, newxyz, harm_module_exec, cgstep, niter, verb=verb )
      call find_phi(newphi,d_est,newmodel,neww)  ! find phi for the new frequency
      alph_it = 0
      alph: do while (newphi > phi - 0.0001*alpha*phi_der*(phi_der/phi_2der) .and. alpha >0.0001)
         alph_it = alph_it + 1
         write(0,*)'line search iteration # ',alph_it,'alpha is ',alpha
         alpha = alpha/2
         neww = w - alpha * (phi_der/phi_2der)
         call data_init(d_est, newxyz, neww)
         call solver_smp( newmodel, newxyz, harm_module_exec, cgstep, niter, verb=verb )
         call find_phi(newphi,d_est,newmodel,neww)
      end do alph
! end line search crap

!begin bisection line search
!      alpha_eps = 10000
!      call data_init( d_est, newxyz, neww )
!      call solver_smp( newmodel, newxyz, harm_module_exec, cgstep, niter, verb=verb )
!      call find_phi( newphi,d_est,newmodel,neww )  ! find phi for the new frequency
!      call find_phi_der( newphi_der, d_est, newmodel, neww )
!!      do while (newphi_der*(phi_der/phi_2der) <=0)  ! the book's way
!      do while (newphi_der <=0 .and. alpha < 100)   ! my way
!         alpha = alpha*2
!         write(0,*)'doubled alpha to ',alpha
!      end do 
!      alph_it = 0
!      alpha_l = 0
!      alpha_u = alpha
!      do while (abs(newphi_der) > alpha_eps)
!         alph_it = alph_it + 1
!         alpha = (alpha_l + alpha_u)/2
!         write(0,*)'line search iteration # ',alph_it,'alpha is ',alpha
!         neww = w - alpha * (phi_der/phi_2der)
!         call data_init(d_est, newxyz, neww)
!         call solver_smp( newmodel, newxyz, harm_module_exec, cgstep, niter, verb=verb )
!         call find_phi(newphi,d_est,newmodel,neww)      ! find phi for the new frequency
!         call find_phi_der(newphi_der, d_est, newmodel, neww)
!         if (newphi_der*(phi_der/phi_2der) > 0) alpha_u = alpha
!         if (newphi_der*(phi_der/phi_2der) < 0) alpha_l = alpha
!      end do 
!
!
!end bisection line search

      err=abs(neww-w)
      write(0,*) 'err is ',err
      write(0,*) ' phi_der is ',phi_der,' phi2_der is ',phi_2der
      write(0,*) 'phi is ',phi
!      write(0,*) 'w is ',w
      if (err>eps .and. abs(phi_der)>der_eps) call L_init(neww)
!      call find_phi_der(phi_der, d_est, model, w)
   end do newton
   ab_mat(:,i2,i3) = model
   f0_vec(1,i2,i3) = w
   f0_vec(2,i2,i3) = iter
!   write(0,*) 'f0 of trace',i2,' ',i3,' is ',w/(2*pi),' Hz' 
end do traces
end do panels

write(0,*)'ab of trace 10 is ',ab_mat(:,10,1)

!subtract the MF
panels2: do i3=1,n3
traces2: do i2=1,n2
!   write(0,*) ' '
!   write(0,*) 'f0 of trace',i2,' ',i3,' is ',f0_vec(1,i2,i3)/(2*pi*d1),' Hz'! .  Iterations: ',f0_vec(2,i2,i3) 
   write(0,*) i2,'& ',f0_vec(1,i2,i3)/(2*pi*d1),'\\\\'   !for output that can go into a latex table...
   !make the harmonic
   ab = ab_mat(:,i2,i3)
!   write(0,*) 'a0 ',ab(1)
!   write(0,*) 'as are ',ab(2:bigM+1)
!   write(0,*) 'bs are ',ab(bigM+2:dim)
   w = f0_vec(1,i2,i3)
!write(0,*) 'w ', w
!   call make_harm( harm, ab, w, n1 )  ! only works for t0s integer multiple of period of dominant freq.
   call make_harm2( harm, ab, w, n1, t0s )
   !subtract the harmonic from the trace
   new_trace = data(:,i2,i3) - harm
!   new_trace = harm
   call sep_write(new_trace)
end do traces2
end do panels2

!output the estimated fundamental frequencies to a file


!deallocate stuff
deallocate ( harm, data, ab_mat, f0_vec, model, xyz, new_trace, d_est )

call sep_close()
call harm_module_close()
call exit(0)
end program Harm_sub
