! ---------------------------------------------------------------------------
module harm_module

use sep
use adj_mod
use matmult

implicit none
integer, private, save         :: bigM, bigN, dim, i1, i2, n
real, private, save            :: d1, d2, o1, o2, f0
real, dimension(:,:), pointer, save :: L, Lf0

contains
! ---------------------------------------------------------------------------
	subroutine harm_module_init( m_in, n_in, f0 )
		integer, intent(IN) :: m_in, n_in
                real                :: f0


		bigM = m_in
		bigN = n_in
		dim = 2*bigM+1

		allocate( L(dim,dim) )
                allocate( Lf0(dim,dim) )

		call L_init(f0)
                Lf0 = L
	end subroutine harm_module_init
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
        subroutine reset_L()

          L=Lf0

        end subroutine reset_L
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
	subroutine L_init(w)
                real,  intent(IN) :: w

	! Initialize L here

          L=0.0
          L(1,1)=1.0

          !gotta loop over N for the sumation of all terms but (1,1) !!!!
        nsum: do n=1, bigN
          axis1: do i1=1, dim
             axis2: do i2=1, dim

             if ((i1 == 1) .and. (i2 > 1) .and. (i2 <= (bigM+1))) then
                L(i1,i2) = L(i1,i2) + sin(w*n*(i2-1))

             else if ((i1 == 1) .and. (i2 > (bigM+1))) then
                L(i1,i2) = L(i1,i2) + cos(w*n*(i2-1-bigM))

             else if ((i2 == 1) .and. (i1 > 1) .and. (i1 <= (bigM+1))) then
                L(i1,i2) = L(i1,i2) + sin(w*n*(i1-1))

             else if ((i2 == 1) .and. (i1 > (bigM+1))) then
                L(i1,i2) = L(i1,i2) + cos(w*n*(i1-1-bigM))

             else if ((i1 > 1) .and. (i2 > 1) .and. (i1 <= (bigM+1)) .and. (i2 <= (bigM+1))) then
                L(i1,i2) = L(i1,i2) + (sin((i1-1)*w*n)*sin((i2-1)*w*n))

             else if ((i1 > 1) .and. (i1 <= (bigM+1)) .and. (i2 > (bigM+1))) then
                L(i1,i2) = L(i1,i2) + (sin((i1-1)*w*n)*cos((i2-1-bigM)*w*n))

             else if ((i2 > 1) .and. (i2 <= (bigM+1)) .and. (i1 > (bigM+1))) then
                L(i1,i2) = L(i1,i2) + sin((i2-1)*w*n)*cos((i1-1-bigM)*w*n)

             else 
                L(i1,i2) = L(i1,i2) + cos((i1-1-bigM)*w*n)*cos((i2-1-bigM)*w*n)
             end if

             end do axis2
          end do axis1
        end do nsum

	end subroutine L_init
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
        subroutine data_init(d_est, xyz, w)
          real, dimension(:), pointer    :: xyz, d_est
          real,  intent(IN)                :: w

!          allocate( xyz(dim) )
!          allocate( d_est(bigN) )
          xyz = 0.0

!write(0,*) 'data_init() - size(d_est,xyz)',size(d_est),size(xyz)
!write(0,*) 'data_init() - min/max(d_est)',minval(d_est),maxval(d_est)
!write(0,*) 'data_init() - min/max(xyz)',minval(xyz),maxval(xyz)

          nsum: do n=1, bigN
             !first element
             xyz(1) = xyz(1) + d_est(n) 
             yz: do i2=1, bigM          
                ! second part- y vector, M terms
                xyz(i2+1) = xyz(i2+1) + (d_est(n) * sin(i2*w*n))
                ! third part- z vector, M term
                xyz(i2+bigM+1) = xyz(i2+bigM+1) + (d_est(n) * cos(i2*w*n))
             end do yz
          end do nsum

!write(0,*) 'data_init() - min/max(d_est)',minval(d_est),maxval(d_est)
!write(0,*) 'data_init() - min/max(xyz)',minval(xyz),maxval(xyz)

	end subroutine data_init
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
	integer function harm_module_exec(adj, add, model, data)
		logical            :: adj, add 
		real, dimension(:) :: model, data 
		integer            :: stat

		call adjnull(adj, add, model, data)

!write(0,*) 'harm_module_exec - TOP'
!write(0,*) 'min/max(L)',minval(L),maxval(L)
		call matmult_init(L)
		stat = matmult_lop(adj,add,model,data)

!                axis1: do i1=1, dim
!                   axis2: do i2=1, dim
!
!                      if( adj) then
!                         data(i1) = data(i1) + L(i1,i2)*model(i2)
!                      else
!                         model(i2) = model(i2) + L(i1,i2)*data(i1)
!                      end if
!
!                   end do axis2
!                end do axis1

		harm_module_exec = 0

!write(0,*) 'harm_module_exec - BOT'

	end function harm_module_exec
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
	subroutine find_phi(phi, d_est, model, w)
          real                        :: phi, next, phi_inc, w
          real, dimension(:), allocatable           ::d_est, model

          phi_inc=0.0
          nsum: do n=1, bigN
             next=d_est(n)
             next=next - model(1)
             sums: do i1=1, bigM
                next = next - model(i1+1)*sin(i1*n*w)
                next = next - model(i1+1+bigM)*cos(i1*n*w)
             end do sums
             phi_inc = phi_inc + next*next
          end do nsum
!          phi = phi_inc * phi_inc    ! needs to be commented, since now we square inside the sumation
          phi = phi_inc
	end subroutine find_phi
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------
	subroutine find_phi_der(phi_der, d_est, model, w)
          real                        :: phi_der, next, w, first, second
          real, dimension(:), allocatable           ::d_est, model

          phi_der=0.0
          nsum: do n=1, bigN
              first = d_est(n)
              first = first - model(1)
              second = 0.
              sums: do i1=1, bigM
                first = first - model(i1+1)*sin(i1*n*w) - model(i1+1+bigM)*cos(i1*n*w)
                second = second - i1*n*model(i1+1)*cos(i1*n*w) + i1*n*model(i1+1+bigM)*sin(i1*n*w)
              end do sums
             next = 2*first*second
             phi_der = phi_der + next
          end do nsum

	end subroutine find_phi_der
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------
	subroutine find_phi_2der(phi_2der, d_est, model, w)
          real                        :: phi_2der, next, w, frst_p1, frst_p2, scnd_p
          real, dimension(:), allocatable           ::d_est, model

          phi_2der=0.0
          nsum: do n=1, bigN
              frst_p1 = d_est(n) - model(1)
              frst_p2 = 0.
              scnd_p = 0.

              sums: do i1=1, bigM
                frst_p1 = frst_p1 - model(i1+1)*sin(i1*n*w) - model(i1+1+bigM)*cos(i1*n*w)
                frst_p2 = frst_p2 + i1*n*i1*n*model(i1+1)*sin(i1*n*w) + i1*n*i1*n*model(i1+1+bigM)*cos(i1*n*w)
                scnd_p = scnd_p - i1*n*model(i1+1)*cos(i1*n*w) + i1*n*model(i1+1+bigM)*sin(i1*n*w)
              end do sums
             next = 2*frst_p1*frst_p2 + 2*scnd_p*scnd_p
             phi_2der = phi_2der + next
          end do nsum

	end subroutine find_phi_2der
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------
	subroutine make_harm( harm, ab, w, n1 )
         real, dimension(:) :: ab, harm
         real                        :: w
         integer                     :: n1

         harm = 0
         samples: do i1=1, n1
            harm(i1) = harm(i1) + ab(1)
            harms: do i2=1, bigM
               harm(i1) = harm(i1) + ab(1+i2)*sin(i2*i1*w) + ab(1+i2+bigM)*cos(i2*i1*w)
            end do harms
         end do samples

!write(0,*) 'Making harmonic...'

         end subroutine make_harm
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------
	subroutine make_harm2( harm, ab, w, n1, t0s )
         real, dimension(:) :: ab, harm
         real                        :: w
         integer                     :: n1, t0s
! this version accounts for the fact that t0 might not be an integer multiple of f0.  


         harm = 0
         samples: do i1=1, n1
            harm(i1) = harm(i1) + ab(1)
            harms: do i2=1, bigM
               harm(i1) = harm(i1) + ab(1+i2)*sin(i2*(i1-t0s+1)*w) + ab(1+i2+bigM)*cos(i2*(i1-t0s+1)*w)
            end do harms
         end do samples

!write(0,*) 'Making harmonic...'

         end subroutine make_harm2
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------
	subroutine harm_module_close()

		if(allocated(L)) then
			deallocate(L)
		else
			call seperr('WARNING: L not allocated in test_module')
		end if

	end subroutine harm_module_close
! ---------------------------------------------------------------------------


end module harm_module
